'use strict';

exports.__esModule = true;
exports.Parallax = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactJss = require('react-jss');

var _reactJss2 = _interopRequireDefault(_reactJss);

var _reactGetNotDeclaredProps = require('react-get-not-declared-props');

var _reactGetNotDeclaredProps2 = _interopRequireDefault(_reactGetNotDeclaredProps);

var _warning = require('warning');

var _warning2 = _interopRequireDefault(_warning);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * A component to render a parallax effect.
 *
 * @class
 * @extends PureComponent
 */
class Parallax extends _react.PureComponent {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.imageIsLoaded = false, this.isIntersecting = false, this.handleIntersection = entries => {
      const entry = entries[0];

      if (!entry.isIntersecting) {
        return;
      }

      if (entry.intersectionRatio === 1 && !this.isIntersecting) {
        window.addEventListener('scroll', this.handleScroll);

        this.isIntersecting = true;
      }

      if (this.isIntersecting && entry.intersectionRatio !== 1) {
        window.removeEventListener('scroll', this.handleScroll);

        this.isIntersecting = false;
      }

      this.positionImage();
    }, this.handleResize = () => {
      this.computeValues();

      this.positionImage();
    }, this.handleScroll = () => {
      this.positionImage();
    }, this.handleImageLoad = () => {
      this.imageIsLoaded = true;

      this.computeValues();

      this.image.style.transform = `translate3D(0, ${-this.overflowImageHeight}px, 0)`;

      this.positionImage();
    }, _temp;
  }

  /**
   * Add the event listener for when the user scrolls.
   */
  componentDidMount() {
    window.addEventListener('resize', this.handleResize);

    this.observer = new IntersectionObserver(this.handleIntersection, { threshold: 1 });

    this.observer.observe(this.root);
  }

  /**
   * Warn against changing the img prop.
   */
  componentWillReceiveProps(nextProps) {
    (0, _warning2.default)(nextProps.img === this.props.img, 'You should not change the img prop of the Parallax');
  }

  /**
   * Remove the event listener again.
   */
  componentWillUnmount() {
    window.removeEventListener('resize', this.handleResize);
    window.removeEventListener('scroll', this.handleScroll);

    this.observer.disconnect();
  }

  /**
   * Compute some static values which don't change when the user scrolls.
   *
   * @private
   */
  computeValues() {
    this.rootHeight = this.root.getBoundingClientRect().height;
    this.imageHeight = this.image.getBoundingClientRect().height;
    this.pixelsToScroll = window.innerHeight - this.rootHeight;
    this.overflowImageHeight = this.imageHeight - this.rootHeight;
  }

  /**
   * Reposition the image based on the current scroll position.
   *
   * @private
   */
  positionImage() {
    if (!this.imageIsLoaded) {
      return;
    }

    var _root$getBoundingClie = this.root.getBoundingClientRect();

    const top = _root$getBoundingClie.top;
    var _window = window;
    const innerHeight = _window.innerHeight;

    const scrollPos = 1 - Math.abs((innerHeight - this.rootHeight - top) / this.pixelsToScroll);
    const transform = scrollPos * this.overflowImageHeight;

    this.image.style.transform = `translate3D(0, ${-transform}px, 0)`;
  }

  /**
   * Recompute the static values and reposition the image.
   *
   * @private
   */


  /**
   * Update the image position on a scroll event.
   * Only update it when the root element is completely visible.
   *
   * @private
   */


  /**
   * When the image loads, compute the values and initially position the image.
   */


  render() {
    var _props = this.props;

    const classes = _props.classes,
          img = _props.img,
          className = _props.className,
          children = _props.children,
          props = _objectWithoutProperties(_props, ['classes', 'img', 'className', 'children']);

    return _react2.default.createElement(
      'div',
      _extends({
        role: 'presentation',
        className: `${classes.root} ${className}`,
        ref: element => {
          this.root = element;
        }
      }, (0, _reactGetNotDeclaredProps2.default)(props, Parallax, 'sheet')),
      _react2.default.createElement('img', {
        width: '100%',
        src: img,
        className: classes.image,
        alt: 'parallax',
        ref: element => {
          this.image = element;
        },
        onLoad: this.handleImageLoad
      }),
      _react2.default.createElement(
        'div',
        { className: classes.content },
        children
      )
    );
  }
}

exports.Parallax = Parallax;
Parallax.propTypes = {
  classes: _propTypes2.default.shape({
    root: _propTypes2.default.string.isRequired,
    image: _propTypes2.default.string.isRequired,
    content: _propTypes2.default.string.isRequired
  }).isRequired,
  img: _propTypes2.default.string.isRequired,
  children: _propTypes2.default.node,
  className: _propTypes2.default.string
};
Parallax.defaultProps = {
  children: '',
  className: ''
};
Parallax.styles = {
  root: {
    composes: 'parallax',
    position: 'relative',
    overflow: 'hidden'
  },

  image: {
    composes: 'parallax--image',
    position: 'absolute',
    top: 0,
    right: 0,
    bottom: 'auto',
    left: 0,
    transition: 'transform 8ms linear',
    zIndex: 0,
    willChange: 'transform'
  },

  content: {
    composes: 'parallax--content',
    zIndex: 1
  }
};
exports.default = (0, _reactJss2.default)(Parallax.styles)(Parallax);